#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <iostream>

#include "KXSC7_2050.h"

KXSC7_2050::KXSC7_2050(mraa_result_t &response, int pin_X, int pin_Y, int pin_Z, double voltage, double r) {
	m_pin_X = pin_X;
	ax = new mraa::Aio(pin_X);
	if (ax == NULL) {
        	response = MRAA_ERROR_UNSPECIFIED;
		return;
    	}
	m_pin_Y = pin_Y;
	ay = new mraa::Aio(pin_Y);
	if (ay == NULL) {
        	response = MRAA_ERROR_UNSPECIFIED;
		return;
    	}
	m_pin_Z = pin_Z;
	az = new mraa::Aio(pin_Z);
	if (az == NULL) {
        	response = MRAA_ERROR_UNSPECIFIED;
		return;
    	}

	_r = r;
	m_voltage = voltage;

	reset();

	response = MRAA_SUCCESS;

	
}

KXSC7_2050::~KXSC7_2050() {
	
	if(ax)delete ax;
	if(ay)delete ay;
	if(az)delete az;
	
}

void KXSC7_2050::setCoefficient(double r)
{
	_r = r;
}

void KXSC7_2050::reset()
{
	const double count = 10;
	double acx, acy, acz;
	last_x = 0;
	last_y = 0;
	last_z = 0;
	for(int i=0;i < count;i++)
	{
		getAccelerationData(acx,acy,acz);
		last_x += acx/count;
		last_y += acy/count;
		last_z += acz/count;
	}
}

void KXSC7_2050::setVoltage(double voltage)
{
	m_voltage = voltage;
	reset();
}

mraa_result_t KXSC7_2050::setPinNum(int pin_X, int pin_Y, int pin_Z)
{
	if(m_pin_X != pin_X)
	{
		if(ax)
		{
			delete ax;
		}
		ax = new mraa::Aio(pin_X);
		if (ax == NULL) {
	        	return MRAA_ERROR_UNSPECIFIED;
	    	}
	}
	m_pin_X = pin_X;

	if(m_pin_Y != pin_Y)
	{
		if(ay)
		{
			delete ay;
		}
		ay = new mraa::Aio(pin_Y);
		if (ay == NULL) {
	        	return MRAA_ERROR_UNSPECIFIED;
	    	}
	}
	m_pin_Y = pin_Y;

	if(m_pin_Z != pin_Z)
	{
		if(az)
		{
			delete az;
		}
		az = new mraa::Aio(pin_Z);
		if (az == NULL) {
	        	return MRAA_ERROR_UNSPECIFIED;
	    	}
	}
	m_pin_Z = pin_Z;

	return MRAA_SUCCESS;
}

void KXSC7_2050::getAccelerationData(double &acx, double &acy, double &acz) {
	float volx = ax->readFloat();
	float voly = ay->readFloat();
	float volz = az->readFloat();
	//std::cout << volx << "\t" << voly << "\t" << volz << std::endl;
	acx = voltage2acceleration(volx);
	acy = voltage2acceleration(voly);
	acz = voltage2acceleration(volz);
	
}

void KXSC7_2050::getAcceleration(double &acx, double &acy, double &acz) {
	getAccelerationData(acx,acy,acz);
	
	
	last_x = (1-_r)*last_x + _r*acx;
	last_y = (1-_r)*last_y + _r*acy;
	last_z = (1-_r)*last_z + _r*acz;


}


double KXSC7_2050::voltage2acceleration(double dVolt) {
  
	
  	const double Vol = 5.0;
	const double g = 9.8;
	const double s = 3.3/0.66;
  	double acc = (dVolt - 0.5*(m_voltage/Vol))*s*g*(Vol/m_voltage);

	return acc;
}